module Authentication
  class CompaniesController < ::ApplicationController
    bmethod :new

    def create
      do_create(params, Authentication::Company, Proc.new { |context, obj| authentication.company_change_path(obj)}, new_params)
    end
    
    bmethod :edit

    def update
      do_update(params, Authentication::Company, main_app.root_path, edit_params)
    end

    def destroy
      skip_policy_scope
      skip_authorization

      @company = ::Authentication::Company.find(params[:id])
      authorize @company
      @company.destroy

      redirect_to main_app.root_path, alert: 'A projekt sikeresen törölve lett, helyette az első projektje lett kiválasztva!'
    end

    bmethod :change do
      bscope { @company = Authentication::Company.find(params[:company_id]) }
      bauth { authorize @company }
      user = Current.user
      user.current_company = @company
      user.save!

      redirect_to main_app.root_path
    end

    bmethod :invite do
      bscope { @company = Authentication::Company.find(params[:company_id]) }
      bauth { authorize @company }
      render 'companies/authentication/invite'
    end

    private

    def before_save_create(obj)
      obj.user = Current.user
      obj.users << Current.user
      Authentication::Company.rights.each do |right|
        Current.user.add_role(right, obj)
      end
    end

=begin    def after_save_create(obj)
      ::Authentication::Company.rights.each do |right|
        Current.user.add_role right, obj
      end
    end
=end
    def new_params
      @object = Company.new

      return [ # model properties
        Btemplater::NewDecorator.new(:name, Proc.new { |f, c| f.input :name, autofocus: true})
      ],
      {
        div: 'div',
        div_class: 'underlined-title',
        # back_url: change_company_path(current_user.current_company?),
        obj: [@object],
        # form_pre: 'shared/form_pre',
        # form_post: 'shared/form_post',
        # form_bottom: 'projects/new_project_type', # optional: renders _edit_extradata.html.erb
        # # submit_button_text: 'Submit'
        form_params: {html: { autocomplete: 'off' }}
      }
    end

    def edit_params
      @object = Authentication::Company.find(params[:id])
      return [ # model properties
       Btemplater::NewDecorator.new(:name, Proc.new { |f, c| f.input :name, autofocus: true})
       ],
       {
        # div: 'div',
        # div_class: 'underlined-title',
        back_url: main_app.root_path,
        # form_pre: 'shared/form_pre',
        form_bottom: 'authentication/company_edit_form_bottom',
        obj: [@object],
        # submit_button_text: 'Submit',
        form_params: {html: { autocomplete: 'off' }}
      }
    end
  end
end
