# frozen_string_literal: true

#
# The locale part of Application Controller
#
module Btemplater::Localization
  extend ActiveSupport::Concern

  included do
    before_action :set_locale

    def set_locale
      supported_locales = I18n.available_locales
      mylocale = nil
      mylocale = Current.user.language unless Current.user.nil?
      locale =
        mylocale || params[:locale] ||
        extract_locale_from_accept_language_header(supported_locales) ||
        I18n.default_locale
      locale = [locale.to_sym] & supported_locales
      I18n.locale = locale.first || supported_locales.first
      logger.debug "* Locale set to '#{I18n.locale}'"
    end

    ##
    # Appends the locale=XX parameter to every URL.
    #
    def default_url_options(options = {})
      logger.debug "default_url_options is passed options: #{options.inspect}\n"
      { locale: I18n.locale }.merge options
    end

    ##
    # Extracts locale from the client.
    #
    def extract_locale_from_accept_language_header(_supported_locales)
      locales = locale_from_request_env.scan(/^[a-z]{2}/).map(&:to_sym)
      # & supported_locales
      locales.first
    end

    def locale_from_request_env
      return '' if request.nil?
      request_env = request.env['HTTP_ACCEPT_LANGUAGE']
      if request_env.nil?
        ''
      else
        request_env
      end
    end
  end
end
