module Btemplater
  module Tools
    def bauthorizer!(env, resource)
      skip_authorization
      raise Pundit::NotAuthorizedError unless env.class.to_s.gsub('Controller', 'Policy').constantize.new(Current, resource).send("#{env.action_name}?")
    end

    def do_create(params, model, redirect_root = nil, new_params = nil)
      if respond_to?(:before_do_create, true)
        send(:before_do_create, @object, params)
      end
      model_name = model.is_a?(Array) ? model.last.to_s : model.to_s
      if model.is_a? Array
        @object = model.first.send(model.last.to_s.demodulize.pluralize.downcase).build(obj_params(@object, model_name, new_params))
      else
        @object = model.new(obj_params(@object, model_name, new_params))
      end

      policy_name = nil
      if new_params[1][:policy_class]
        policy_name = "#{new_params[1][:policy_class]}Policy".constantize
      else
        policy_name = "#{@object.class.name.to_s}Policy".constantize
      end
      security_items = new_params[1][:security_items] || @object

      skip_policy_scope
      skip_authorization

      # authorize (security_items), :create?
      raise Pundit::NotAuthorizedError unless(policy_name.new(Current, security_items).create?)
      # send(:before_save_create, @object, params) if respond_to?(:before_save_create, true)
      if respond_to?(:before_save_create, true)
        arg_num = method(:before_save_create).parameters.count
        if arg_num == 1
          send(:before_save_create, @object)
        elsif arg_num == 2
          send(:before_save_create, @object, params)
        end
      end
      # before_save_create if method_defined? :before_save_create
      if @object.save
        flash['notice'] = new_params[1][:flash_success_message] || t('btemplater.controllers.create_success')
        if redirect_root.nil?
          send(:after_save_create, @object) if respond_to?(:after_save_create, true)
          redirect_to polymorphic_url new_params[1][:obj]
        # elsif redirect_root.is_a? Proc
        #   send(:after_save_create, @object) if respond_to?(:after_save_create, true)
        #   redirect_to redirect_root.call(self, @object)
        else
          send(:after_save_create, @object) if respond_to?(:after_save_create, true)
          if redirect_root.is_a? Proc
            redirect_to redirect_root.call(self, @object)
          else
            redirect_to redirect_root
          end
        end
      else
        # render 'new'
        # new_params[2][:obj] = model
        render inline: supernew(new_params[0], new_params[1], view_context), layout: true
      end
    end

    def do_update(params, model, redirect_root = nil, edit_params = nil)
      @object = model.find(params[:id])
      policy_name = nil
      if edit_params[1][:policy_class]
        policy_name = "#{edit_params[1][:policy_class]}Policy".constantize
      else
        policy_name = "#{model.to_s}Policy".constantize
      end
      security_items = edit_params[1][:security_items] || @object

      skip_policy_scope
      skip_authorization

      # authorize (security_items), :update?
      raise Pundit::NotAuthorizedError unless(policy_name.new(Current, security_items).update?)
      if respond_to?(:before_save_update, true)
        arg_num = method(:before_save_update).parameters.count
        if arg_num == 1
          send(:before_save_update, @object)
        elsif arg_num == 2
          send(:before_save_update, @object, params)
        end
      end
      # before_save_create if method_defined? :before_save_create
      if @object.update(obj_params(@object, model, edit_params))
        flash['notice'] = t('btemplater.controllers.update_success')
        if redirect_root.nil?
          redirect_to @object
        elsif redirect_root.is_a? Proc
          send(:after_save_update, @object) if respond_to?(:after_save_update, true)
          redirect_to redirect_root.call(self, @object)
        else
          redirect_to redirect_root
        end
      else
        render inline: superedit(edit_params[0], edit_params[1], view_context), layout: true
      end
    end

    def do_destroy(params, model, redirect_root = nil)
      @object = model.find(params[:id])
      policy_name = nil
      # if args[1][:policy_class]
        # policy_name = "#{args[1][:policy_class]}Policy".constantize
      # else
      policy_name = "#{model.to_s}Policy".constantize
      # end
      authorize (@object), :destroy?
      raise Pundit::NotAuthorizedError unless(policy_name.new(Current, @object).destroy?)
      @object.destroy
      if @object.destroyed?
        send(:after_destroy, @object) if respond_to?(:after_destroy, true)
        flash['notice'] = t('btemplater.controllers.destroy_success')
      end
      if redirect_root.nil?
        redirect_to @object
      else
        redirect_to redirect_root
      end
    end

    private

    def obj_params(obj, model, args = {})
      mdl = model
      if model.is_a? Array
        mdl = model.last
      end

      policy_name = nil
      if args[1][:policy_class]
        policy_name = "#{args[1][:policy_class]}Policy".constantize
      else
        policy_name = "#{mdl.to_s}Policy".constantize
      end
      policy = policy_name.new(Current, obj || model)
      params.require(model.to_s.demodulize.underscore.to_sym).permit(policy.permitted_attributes)
    end
  end
end
